import { Pushable } from 'it-pushable';
import { Message } from './message-types.js';
import type { AbortOptions } from '@libp2p/interfaces';
import type { Sink } from 'it-stream-types';
import type { Muxer } from '@libp2p/interfaces/stream-muxer';
import type { Stream } from '@libp2p/interfaces/connection';
import type { ComponentMetricsTracker } from '@libp2p/interfaces/metrics';
export interface MplexStream extends Stream {
    source: Pushable<Uint8Array>;
}
export interface MplexOptions extends AbortOptions {
    onStream?: (...args: any[]) => void;
    onStreamEnd?: (...args: any[]) => void;
    maxMsgSize?: number;
    metrics?: ComponentMetricsTracker;
}
export declare class Mplex implements Muxer {
    static multicodec: string;
    sink: Sink<Uint8Array>;
    source: AsyncIterable<Uint8Array>;
    private _streamId;
    private readonly _streams;
    private readonly _options;
    private readonly _source;
    constructor(options?: MplexOptions);
    /**
     * Returns a Map of streams and their ids
     */
    get streams(): Stream[];
    /**
     * Initiate a new stream with the given name. If no name is
     * provided, the id of the stream will be used.
     */
    newStream(name?: string): Stream;
    /**
     * Called whenever an inbound stream is created
     */
    _newReceiverStream(options: {
        id: number;
        name: string;
    }): MplexStream;
    _newStream(options: {
        id: number;
        name: string;
        type: 'initiator' | 'receiver';
        registry: Map<number, MplexStream>;
    }): MplexStream;
    /**
     * Creates a sink with an abortable source. Incoming messages will
     * also have their size restricted. All messages will be varint decoded.
     */
    _createSink(): Sink<Uint8Array, Promise<void>>;
    /**
     * Creates a source that restricts outgoing message sizes
     * and varint encodes them
     */
    _createSource(): AsyncGenerator<Uint8Array, void, undefined> & {
        push: (value: Message) => import("it-pushable").PushableV<Message>;
        end: (err?: Error | undefined) => import("it-pushable").PushableV<Message>;
        return: () => {
            done: boolean;
        };
    };
    _handleIncoming(message: Message): void;
}
//# sourceMappingURL=index.d.ts.map